/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.util.Random;

/**
 * Abstract implementation of IDictionary.
 * 
 * @author Jacek Koleyski
 */
public abstract class AbstractFileDictionary implements IDictionary {
	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------

	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------
	/** The file name. */
	protected String filename;

	/** The wordsreminder object. */
	protected Reminder reminder;

	/** Encoding used. */
	protected String encoding;

	/** Random generator */
	protected Random random = new Random();

	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------

	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------
	/**
	 * Creates a new object.
	 * 
	 * @param fileName
	 *            the file name
	 * @param wordsreminder
	 *            the Reminder object
	 */
	public AbstractFileDictionary(Reminder reminder, String filename,
			String encoding) {
		this.filename = filename;
		this.encoding = encoding;
		this.reminder = reminder;
	}

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------
	/**
	 * Adds a word to the dictionary.
	 * 
	 * @param wordWithTranslation
	 *            word with translation
	 * 
	 * @return the ITerm created
	 */
	public ITerm addToDictionary(String wordWithTranslation) {
		try {
			return addToDictionary(Utils.createTerm(wordWithTranslation, -1));
		} catch (DictionaryCorruptedException e) {
			reminder.showMessage(reminder.badFormatString1 + e.getMessage()
					+ reminder.badFormatString2, false);

			return null;
		}
	}
}
