/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.io.File;

/**
 * A factory class, creating IDictionary implementation.
 * 
 * @author Jacek Koleyski
 */
public class DictionaryFactory {
	// ~ Static fields/initializers
	// -------------------------------------------------------------------------------------

	// ~ Static fields/initializers
	// -------------------------------------------------------------------------------------
	/** The limit of kilobytes. */
	public static int LIMIT_KB = 300000;

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Creates a new FileDictionary object from file.
	 * 
	 * @param reminder
	 *            the Reminder object
	 * @param filename
	 *            the file name
	 * @param encoding
	 *            the encoding used
	 * 
	 * @return IDictionary implementation
	 */
	public static IDictionary getDictionary(Reminder reminder, String filename,
			String encoding) {
		File f = new File(filename);

		if (f.length() < LIMIT_KB) {
			return FileDictionary.readDictionaryFromFile(reminder, filename,
					encoding);
		} else {
			return VeryBigFileDictionary.readDictionaryFromFile(reminder,
					filename, encoding);
		}
	}

	/**
	 * Creates an empty FileDictionary.
	 * 
	 * @param reminder
	 *            the Reminder object
	 * @param filename
	 *            the file name
	 * @param encoding
	 *            the encoding used
	 * 
	 * @return IDictionary implementation
	 */
	public static IDictionary createEmptyDictionary(Reminder reminder,
			String filename, String encoding) {
		return FileDictionary.createEmptyDictionary(reminder, filename,
				encoding);
	}
}
