/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * Extended RandomAccessFile class, adding support for using specified encoding
 * while reading lines.
 * 
 * @author Jacek Koleyski
 */
public class ExtendedRandomAccessFile extends RandomAccessFile {
	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------

	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------
	/**
	 * {@inheritDoc}
	 */
	public ExtendedRandomAccessFile(File arg0, String arg1)
			throws FileNotFoundException {
		super(arg0, arg1);
	}

	/**
	 * {@inheritDoc}
	 */
	public ExtendedRandomAccessFile(String arg0, String arg1)
			throws FileNotFoundException {
		super(arg0, arg1);
	}

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------
	/**
	 * Reads the line using specified encoding.
	 * 
	 * @param encoding
	 *            the encoding to be used
	 * 
	 * @return line decoded using the specified encoding
	 * 
	 * @throws IOException
	 *             thrown if something goes wrong
	 */
	public final String readLine(String encoding) throws IOException {
		int tableLength = 80;
		int currentPos = 0;
		byte[] bytes = new byte[tableLength];
		byte b = -1;
		boolean eol = false;

		while (!eol) {
			switch (b = readByte()) {
			case -1:
			case '\n':
				eol = true;

				break;

			case '\r':
				eol = true;

				long cur = getFilePointer();

				if ((readByte()) != '\n') {
					seek(cur);
				}

				break;

			default:

				if (currentPos < tableLength) {
					bytes[currentPos] = b;
				} else {
					int oldTableLength = tableLength;
					tableLength = tableLength * 2;

					byte[] newBytes = new byte[tableLength];
					System.arraycopy(bytes, 0, newBytes, 0, oldTableLength);
					bytes = newBytes;
					bytes[currentPos] = b;
				}

				currentPos++;

				break;
			}
		}

		if ((b == -1) && (currentPos == 0)) {
			return null;
		}

		return new String(bytes, encoding);
	}
}
