/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.util.ArrayList;
import java.util.List;

/**
 * This class performs a history of used terms.
 * 
 * @author Jacek Koleyski
 * 
 * @param <T>
 *            the type of object to be stored
 */
public class HistoryCollection<T extends Object> {
	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------

	// ~ Static fields/initializers
	// -------------------------------------------------------------------------------------
	/** Maximum size. */
	private int maxSize = 15;

	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------
	/** List of objects in history. */
	private List<T> list = new ArrayList<T>();

	// ~ Constructors
	// ---------------------------------------------------------------------------------------------------

	/**
	 * Creates a new HistoryCollection object.
	 * 
	 * @param maxSize
	 *            maximum size of the collection
	 */
	public HistoryCollection(int maxSize) {
		super();
		this.maxSize = maxSize;
	}

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Adds a term to the dictionary.
	 * 
	 * @param t
	 *            the term
	 */
	public void addToHistory(T t) {
		if (list.size() == maxSize) {
			list.remove(maxSize - 1);
		}

		list.add(0, t);
	}

	/**
	 * Gets an object from the history list.
	 * 
	 * @param i
	 *            number of the object
	 * 
	 * @return object from the history
	 */
	public T get(int i) {
		return list.get(i);
	}

	/**
	 * Gets a list of all objects in history.
	 * 
	 * @return the list of objects in history
	 */
	public List<T> getList() {
		return list;
	}

	/**
	 * Returns the size of the history list.
	 * 
	 * @return the size of the history list
	 */
	public int size() {
		return list.size();
	}
}
