/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

/**
 * Interface of a term. Represents a word with its translations.
 * 
 * @author Jacek Koleyski
 */
public interface ITerm {
	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------
	/**
	 * Checks if the given translation matches one of the translations
	 * configured for this term.
	 * 
	 * @param translation
	 *            the translation to be checked
	 * 
	 * @return true if the translation is ok, false if not
	 */
	public boolean checkTranslation(String translation);

	/**
	 * Gets a table of translations.
	 * 
	 * @return table containing all translations
	 */
	public String[] getTranslation();

	/**
	 * Gets translations as one concatenated String.
	 * 
	 * @return concatenated String of all translations
	 */
	public String getTranslationAsOneString();

	/**
	 * Gets the word of the term.
	 * 
	 * @return word of the term
	 */
	public String getWord();

	/**
	 * Joins the term with another term.
	 * 
	 * @param term
	 *            term to be joined
	 * 
	 * @return new joined term
	 */
	public ITerm join(ITerm term);
}
