/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Properties;
import java.util.Random;
import java.util.Scanner;
import java.util.StringTokenizer;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.MenuElement;
import javax.swing.MenuSelectionManager;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

/**
 * The main class of the program.
 * 
 * @author Jacek Koleyski
 */
public class Reminder extends JFrame {
	/** Current version. */
	static final String currentVersion = "1.5.3";

	/** About message */
	static final String ABOUT_MESSAGE = "Words Reminder (Odpytywacz Swek), v "
			+ currentVersion
			+ " \nCopyright (C) 2007, Jacek Koleyski\n\n"
			+ "The official site: http://code.google.com/p/wordsreminder/\n"
			+ "Check it regularly for updates.\n\n"
			+ "This program is free software licensed under the GPL license. \n"
			+ "But if you use it, please send me a card or an e-mail: jacek.kolezynski@gmail.com\n"
			+ "You can also donate, if you want, a dollar or two (for a beer :-), "
			+ "here is my account number: 50102055581111114270200257\n\n"
			+ "Ten program jest darmowy, na licencji GPL.\n"
			+ "Jesli jednak go uywasz, przylij mi kartke lub e-mail: jacek.kolezynski@gmail.com\n"
			+ "Moesz take, jeli chcesz, dorzuci si dla mnie na piwo :-), "
			+ "to jest numer mojego rachunku: 50102055581111114270200257\n\n";

	/**
	 * Value used for specifying if many words should be added in a row, or just
	 * one.
	 */
	private static final int ADD_MANY_WORDS = 0;

	/**
	 * Value used for specifying that currently asked question is of first
	 * language.
	 */
	private static final int FIRST_LANGUAGE = 1;

	/**
	 * Value used for specifying that currently asked question is of second
	 * language.
	 */
	private static final int SECOND_LANGUAGE = 2;

	/** Value used for specifying that currently asked question is of extended. */
	private static final int EXTENDED = 3;

	/** Value specifying maximum size of history collection. */
	private static final int MAX_HISTORY_SIZE = 20;

	/** Term separator. */
	public static String termSeparator = "=";

	/** Separator used in extended. */
	public static String extendedSeparatorFirst = "==";

	/** Separator used in extended. */
	public static String extendedSeparatorSecond = "::";

	/** Translations separator. */
	public static String translationSeparator = ";";

	/** Localized string for line number error text. */
	static String lineNrString;

	/** Localized string for about menu. */
	private String aboutMenuString;

	/** Localized string for usage menu. */
	private String usageMenuString;

	/** About menu item. */
	private JMenuItem aboutMI;

	/** Usage menu item. */
	private JMenuItem usageMI;

	/** Usage menu item. */
	private JMenuItem checkUpdatesMI;

	/** Actions button. */
	private JButton menuButton;

	/** Flag used to prevent showing input dialog twice (a trick). */
	private boolean addingToDictionary;

	/** Add to dictionary menu item */
	private JMenuItem addToDictionaryMI;

	/** The mode of adding to dictionary, default many words. */
	private int addToDictionaryMode = ADD_MANY_WORDS;

	/** Localized string for add to dictionary menu. */
	private String addToDictionaryString;

	/** Always on top menu item. */
	private JCheckBoxMenuItem alwaysOnTopMI;

	/** Center window menu item. */
	private JCheckBoxMenuItem centerMI;

	/** Localized string for always on top menu. */
	private String alwaysOnTopString;

	/** The text field to enter answers and show help. */
	private TimerTextField answerTextField = new TimerTextField(22);

	/** Localized string for ask both languages menu. */
	private String askBothLanguagesString;

	/** The delay to show help. After this time new question will be asked. */
	private int helpDuration = 3;

	/** Localized string for ask only first language menu. */
	private String askOnlyFirstLanguageString;

	/** Localized string for ask only second language menu. */
	private String askOnlySecondLanguageString;

	/** Localized string for bad format message 1. */
	String badFormatString1;

	/** Localized string for bad format message 2. */
	String badFormatString2;

	/** Localized string for bad answer. */
	private String badAnswerString;

	/** Localized string for bad answer. */
	private String badAnswerString2;

	/** Localized string for no extended files. */
	private String noExtendedFilesErrorString;

	/** Ask of both languages menu item. */
	private JRadioButtonMenuItem bothLanguagesMI;

	/** Localized string for change language menu. */
	private String changeLanguageMenuString;

	/** Localized string for coding UTF8 menu. */
	private String codingUTF8String;

	/** Localized string for coding Windows1250 menu. */
	private String codingWindows1250String;

	/** Name of config file. */
	private String configFileName = "config.properties";

	/** Localized string for loading configuration error. */
	private String configurationLoadingError;

	/** Localized string for correct answers text. */
	private String correctAnswersString;

	/** Encoding Windows1250 menu item. */
	private JMenuItem cp1250MI;

	/** Current question displayed. */
	private String currentQuestion;

	/** The current term used. */
	private ITerm currentTerm;

	/** Dictionary used. */
	private IDictionary dictionary;

	/** Dictionary used. */
	private IDictionary alreadyLearntDictionary;

	/** Dictionary of frequent vocabulary terms. */
	private IDictionary frequentVocabularyDictionary;

	/** Localized string for dictionary file error 1. */
	String dictionaryFileErorString1;

	/** Localized string for dictionary file error 2. */
	String dictionaryFileErorString2;

	/** Localized string for empty dictionary situation. */
	private String dictionaryIsEmptyErrorString;

	/** Default dictionary file name. */
	private String dictionaryFileName = "dictionary.txt";

	/** Font used in question label and text field. */
	private Font textFont;

	/** Encoding used. */
	private String encoding = "UTF-8";

	/** Localized string for enter term example. */
	private String enterTermExampleString;

	/** Localized string for enter term message. */
	private String enterTermString;

	/** Exit menu item. */
	private JMenuItem exitMI;

	/** Localized string for exit menu. */
	private String exitString;

	/** File menu. */
	private JMenu fileMenu;

	/** Localized string for file menu. */
	private String fileMenuString;

	/** Ask of first language only menu item. */
	private JRadioButtonMenuItem firstLanguageMI;

	/** Localized string for good answer. */
	private String goodAnswerString;

	/** Localized string for good answer. */
	private String goodAnswerString2;

	/** Help button. */
	private JButton helpButton;
	
	/** Pause button. */
	private JButton pauseButton;

	/** Help menu. */
	private JMenu helpMenu;

	/** Localized string for help menu. */
	private String helpMenuString;

	/** Help task. */
	private HelpTask helpTask;

	/** History of all terms. */
	private HistoryCollection<ITerm> historyVocabulary = new HistoryCollection<ITerm>(
			MAX_HISTORY_SIZE);

	/** History menu. */
	private JMenu historyVocabularyMenu;

	/** Localized string for history menu. */
	private String historyVocabularyMenuString;

	/** History menu items. */
	private TermMenuItem[] historyVocabularyMI;

	/** The image used as the application icon. */
	private Image image;

	/** The image used as the application icon in messages. */
	private ImageIcon imageIcon;

	/** Localized string for incorrect answers text. */
	private String incorrectAnswersString;

	/** Flag telling if the GUI language is english. */
	private boolean languageIsEnglish = true;

	/** Flag telling if the window is always centered. */
	private boolean center = true;

	/** Choose language of GUI menu item. */
	private JCheckBoxMenuItem languageMI;

	/** Last path in file chooser. */
	private String lastPath = ".";

	/** Load dictionary menu item. */
	private JMenuItem loadDictionaryMI;

	/** Localized string for load dictionary menu. */
	private String loadDictionaryString;

	/** Menu bar. */
	private JMenuBar menuBar;

	/** Flag telling if menu bar is visible. */
	private boolean menuBarVisible;

	/** Options menu. */
	private JMenu optionsMenu;

	/** Localized string for options menu. */
	private String optionsMenuString;

	/** The question label. */
	private JLabel questionLabel = new JLabel("");

	/** Random generator used for generating random questions. */
	private Random random = new Random();

	/** Flag telling if the refreshing takes place now. */
	private boolean refreshing;

	/** Refresh task. */
	private TimerTask refreshTask;

	/** Counter all questions. */
	private int questionsAsked;

	/** Counter of right answers. */
	private int rightAnswers;

	/** Localized string for save changes menu. */
	private String saveChangesString;

	/** Save changes menu item. */
	private JMenuItem saveMI;

	/** Ask of second language menu item. */
	private JRadioButtonMenuItem secondLanguageMI;

	/** Localized string for show help after 20 seconds menu. */
	private String showHelpAfterXSecondsString;

	/** Localized string for help duration. */
	private String helpDurationXString;

	/** After this delay show will be showed, if enabled. */
	private int showHelpDelay = 20;

	/** Show no help menu item. */
	private JMenuItem showHelpNowMI;

	/** Show extended question now menu item. */
	private JMenuItem showExtendedQuestionNowMI;

	/** Localized string for show help now menu. */
	private String showExtendedQuestionNowString;

	/** Localized string for show help now menu. */
	private String showHelpNowString;

	/** Localized string for show no help at all menu. */
	private String showNoHelpString;

	/** This frame reference. */
	public JFrame thisFrame = this;

	/** Localized string for center position menu. */
	private String centerString;

	/** Timer for tasks scheduling. */
	private Timer timer = new Timer();

	/** Localized string for title. */
	private String titleString;

	/** Localized string for total answered questions text. */
	private String totalAnsweredQuestionsString;

	/** Encoding UTF8 menu item. */
	private JMenuItem utf8MI;

	/** Dictionary file name from argument list */
	private String wantedDictionaryFileName;

	/** Flag telling which language is used now. */
	private int currentQuestionType = FIRST_LANGUAGE;

	/** Counter of wrong answers. */
	private int wrongAnswers;

	/** Filename filter used for extended files, txt. */
	private FilenameFilter filenameFilter;

	/** Flag indicating if extended question is displayed now. */
	private boolean grammarQuestionOngoing;

	/** Flag indicating if extended question is displayed now. */
	private boolean sentenceQuestionOngoing;

	/** Localized string for check extended answer. */
	private String checkExtendedString;

	/** Localized string for help extended. */
	private String helpExtendedString;

	/** Localized string for next extended question. */
	private String nextExtendedString;

	/** Localized string for exit from extended. */
	private String exitExtendedString;

	/** Localized string for extended questions frequency. */
	private String extendedQuestionFrequencyString;

	/** Slider used for setting extended questions frequency. */
	private SliderMenuItem extendedQuestionFrequencySMI;

	/** Slider used for setting the delay of showing help. */
	private SliderMenuItem showHelpDelaySMI;

	/** Slider used for setting the delay of showing help. */
	private SliderMenuItem helpDurationSMI;

	/** Value for show extended questions delay. */
	private int showExtendedQuestionDelay = 0;

	/** The task used for displaying extended questions periodically. */
	private TimerTask showExtendedQuestionTimerTask;

	/** Localized string for not showing extended at all. */
	private String notShowExtendedQuestionsString;

	/** Localized string for too many already-learnt words */
	private String tooManyAlreadyLearntLongInfo;

	/** Extended exit button. */
	private JButton extendedExitBtn;

	/** Extended help button. */
	private JButton extendedHelpBtn;

	/** Extended next button. */
	private JButton extendedNextBtn;

	/** Extended check button. */
	private JButton extendedCheckBtn;

	/** Localized string for asking to update to new version. */
	private String updateToNewVersionString;

	/** Localized string saying the application is up to date. */
	private String applicationIsUpToDateString;

	/** Localized string saying that it is not possible to check for updates. */
	private String cannotPerforUpToDateCheckString;

	/** Localized string for check for updates menu item. */
	private String checkForUpdatesMenuItemString;

	/** Current sentences file name. */
	private String currentSentecesFileName;

	/** Sentences dictionary. */
	private IDictionary sentencesDictionary;

	/** The date of the last update check. */
	private Date lastUpdateCheck;

	/** Position of hint */
	private int currentHintPosition;

	private String helpTooltipString;

	private Font iconFont = new Font("Courier New", Font.BOLD, 32);

	private TextFieldMenuItem extendedQuestionCategory;

	private String extendedQuestionCategoryFieldTitle;

	private String extendedQuestionCategoryTooltip;

	/**
	 * Creates a new Reminder object.
	 */
	public Reminder(String filename) {
		super();

		getLanguageBundle(languageIsEnglish);
		wantedDictionaryFileName = filename;
		setIcon(0);
		setResizable(false);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent arg0) {
				System.exit(0);
			}
		});

		textFont = questionLabel.getFont().deriveFont(18.0f);
		questionLabel.setFont(textFont);
		// questionLabel.setPreferredSize(new Dimension(250,25));
		answerTextField.setFont(textFont);

		setUndecorated(true);

		menuBar = new JMenuBar();
		menuBar.setVisible(menuBarVisible);
		fileMenu = new JMenu(fileMenuString);
		optionsMenu = new JMenu(optionsMenuString);
		menuBar.add(fileMenu);
		menuBar.add(optionsMenu);

		// Create and add a fileMenu item.
		loadDictionaryMI = new JMenuItem(loadDictionaryString);
		loadDictionaryMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadDictionary(false);
			}
		});

		fileMenu.add(loadDictionaryMI);

		// Create and add a fileMenu item.
		addToDictionaryMI = new JMenuItem(addToDictionaryString);
		addToDictionaryMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addToDictionary();
			}
		});
		addToDictionaryMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
				InputEvent.CTRL_MASK));
		fileMenu.add(addToDictionaryMI);

		// Create and add a fileMenu item.
		saveMI = new JMenuItem(saveChangesString);
		saveMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveConfiguration();
			}
		});
		saveMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				InputEvent.CTRL_MASK));
		fileMenu.add(saveMI);

		exitMI = new JMenuItem(exitString);
		exitMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveConfiguration();
				showMessage(totalAnsweredQuestionsString
						+ (rightAnswers + wrongAnswers) + "\n"
						+ correctAnswersString + " " + rightAnswers + "\n"
						+ incorrectAnswersString + " " + wrongAnswers, true);
			}
		});
		exitMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
				InputEvent.CTRL_MASK));
		fileMenu.add(exitMI);
		setJMenuBar(menuBar);

		// ----- OPTIONS menu
		// ---------------------------------------------------------------------------
		languageMI = new JCheckBoxMenuItem(changeLanguageMenuString);
		languageMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				languageMI.setSelected(false);
				getLanguageBundle(!languageIsEnglish);
				refreshGui();
			}
		});
		optionsMenu.add(languageMI);

		alwaysOnTopMI = new JCheckBoxMenuItem(alwaysOnTopString, true);
		alwaysOnTopMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setAlwaysOnTop(alwaysOnTopMI.isSelected());
			}
		});
		optionsMenu.add(alwaysOnTopMI);

		// ----- centering settings
		// -------------------------------------------------------------
		centerMI = new JCheckBoxMenuItem(centerString);
		centerMI.setSelected(true);
		centerMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				center = centerMI.isSelected();
			}
		});
		optionsMenu.add(centerMI);
		optionsMenu.addSeparator();

		// ----- show help now settings
		// -------------------------------------------------------------
		showHelpNowMI = new JMenuItem(showHelpNowString);

		char ch = '/';
		showHelpNowMI.setAccelerator(KeyStroke.getKeyStroke(ch));
		showHelpNowMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				help();
			}
		});
		optionsMenu.add(showHelpNowMI);
		optionsMenu.addSeparator();

		showExtendedQuestionNowMI = new JMenuItem(showExtendedQuestionNowString);

		// ----- show extended question now settings
		// -------------------------------------------------------------
		ch = '.';
		showExtendedQuestionNowMI.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_E, InputEvent.CTRL_MASK));
		showExtendedQuestionNowMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				askExtendedQuestion();
			}
		});

		optionsMenu.add(showExtendedQuestionNowMI);
		optionsMenu.addSeparator();
		extendedQuestionCategory = new TextFieldMenuItem(
				extendedQuestionCategoryFieldTitle, "", thisFrame
						.getBackground());
		extendedQuestionCategory
				.setToolTipText(extendedQuestionCategoryTooltip);
		optionsMenu.add(extendedQuestionCategory);

		// ----- extended question settings
		// -------------------------------------------------------------
		extendedQuestionFrequencySMI = new SliderMenuItem(0, 60,
				showExtendedQuestionDelay, extendedQuestionFrequencyString);
		extendedQuestionFrequencySMI.setForeground(Color.red);
		extendedQuestionFrequencySMI.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if (extendedQuestionFrequencySMI.getValueIsAdjusting()) {
					return;
				}

				showExtendedQuestionDelay = extendedQuestionFrequencySMI
						.getValue();

				if (showExtendedQuestionTimerTask != null) {
					showExtendedQuestionTimerTask.cancel();
				}

				if (showExtendedQuestionDelay == 0) {
					extendedQuestionFrequencySMI.setForeground(Color.red);
					extendedQuestionFrequencySMI
							.setTitle(notShowExtendedQuestionsString);
				} else {
					extendedQuestionFrequencySMI.setForeground(Color.blue);
					extendedQuestionFrequencySMI
							.setTitle(extendedQuestionFrequencyString
									.replaceAll(
											"X",
											Integer
													.toString(showExtendedQuestionDelay)));
					scheduleExtendedQuestion();
				}
			}
		});

		if (showExtendedQuestionDelay > 0) {
			extendedQuestionFrequencySMI
					.setTitle(extendedQuestionFrequencyString.replaceAll("X",
							Integer.toString(showExtendedQuestionDelay)));
		} else {
			extendedQuestionFrequencySMI
					.setTitle(notShowExtendedQuestionsString);
		}

		if (showExtendedQuestionDelay > 0) {
			scheduleExtendedQuestion();
		}

		optionsMenu.add(extendedQuestionFrequencySMI);
		optionsMenu.addSeparator();

		// ----- language settings
		// ----------------------------------------------------------------------
		ButtonGroup group = new ButtonGroup();
		firstLanguageMI = new JRadioButtonMenuItem(askOnlyFirstLanguageString);
		group.add(firstLanguageMI);
		optionsMenu.add(firstLanguageMI);

		secondLanguageMI = new JRadioButtonMenuItem(askOnlySecondLanguageString);
		group.add(secondLanguageMI);
		optionsMenu.add(secondLanguageMI);

		bothLanguagesMI = new JRadioButtonMenuItem(askBothLanguagesString);
		bothLanguagesMI.setSelected(true);
		group.add(bothLanguagesMI);
		optionsMenu.add(bothLanguagesMI);
		optionsMenu.addSeparator();

		// ----- show help delay settings
		// ---------------------------------------------------------------
		ShowHelpChangeListener shcl = new ShowHelpChangeListener();
		showHelpDelaySMI = new SliderMenuItem(0, 60, 20,
				showHelpAfterXSecondsString);
		showHelpDelaySMI.addChangeListener(shcl);
		showHelpDelaySMI.setForeground(Color.blue);

		if (showHelpDelay > 0) {
			showHelpDelaySMI.setTitle(showHelpAfterXSecondsString.replaceAll(
					"X", Integer.toString(showHelpDelay)));
		} else {
			showHelpDelaySMI.setTitle(showNoHelpString);
		}

		optionsMenu.add(showHelpDelaySMI);
		optionsMenu.addSeparator();

		// ----- help duration settings
		// ---------------------------------------------------------------
		HelpDurationChangeListener hdcl = new HelpDurationChangeListener();
		helpDurationSMI = new SliderMenuItem(0, 60, 3, helpDurationXString);
		helpDurationSMI.addChangeListener(hdcl);
		helpDurationSMI.setForeground(Color.blue);

		helpDurationSMI.setTitle(helpDurationXString.replaceAll("X", Integer
				.toString(helpDuration)));

		optionsMenu.add(helpDurationSMI);
		optionsMenu.addSeparator();

		// ----- encoding settings
		// ----------------------------------------------------------------------
		ActionListener encodingAL = new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				String enc = "";

				if (utf8MI.isSelected()) {
					enc = "UTF-8";
				} else {
					enc = "Cp1250";
				}

				if (!enc.equals(encoding)) {
					encoding = enc;

					getDictionary(false);
				}
			}
		};

		group = new ButtonGroup();
		utf8MI = new JRadioButtonMenuItem(codingUTF8String);
		group.add(utf8MI);
		utf8MI.setSelected(true);
		utf8MI.addActionListener(encodingAL);
		optionsMenu.add(utf8MI);

		cp1250MI = new JRadioButtonMenuItem(codingWindows1250String);
		group.add(cp1250MI);
		cp1250MI.addActionListener(encodingAL);
		optionsMenu.add(cp1250MI);

		// ----- help buttton settings
		// ------------------------------------------------------------------
		helpButton = new JButton("?");
		helpButton.setToolTipText(helpTooltipString);
		helpButton.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (SwingUtilities.isLeftMouseButton(e)) {
					help();
				} else {
					hint();
				}
			}
		});
		
		// ----- pause buttton settings
		// ------------------------------------------------------------------
		pauseButton = new JButton("II");
		pauseButton.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				   if(helpTask.isPaused())
					   helpTask.pause(0);
				   else
					   helpTask.pause(Integer.MAX_VALUE);
				}
		});

		// ----- history settings
		// -----------------------------------------------------------------------
		historyVocabularyMenu = new JMenu(historyVocabularyMenuString);
		menuBar.add(historyVocabularyMenu);
		historyVocabularyMI = new TermMenuItem[MAX_HISTORY_SIZE];

		MouseListener historyVocabularyListener = new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				// first get the term
				ITerm theTerm = ((TermMenuItem) e.getSource()).getTerm();

				// if it was a normal left click, it is the action to add the
				// term to the frequent
				// terms
				if (SwingUtilities.isLeftMouseButton(e)) {
					// but first we need to check if it is somehow in the
					// already-learnt terms
					// if it is, then we do nothing
					if (alreadyLearntDictionary.contains(theTerm)) {
						return;
					}

					// if it is not in the already-learnt terms, then it is ok
					// to put it into the
					// frequent terms
					if (frequentVocabularyDictionary.contains(theTerm)) {
						frequentVocabularyDictionary
								.removeFromDictionary(theTerm);
					} else {
						frequentVocabularyDictionary.addToDictionary(theTerm);
					}
				} else {
					// here we have a right click, so it is an action to add the
					// term to the
					// already-learnt terms
					// but first we need to check if the term is one of the
					// frequent terms,
					// because if it is, we first need to remove it from the
					// frequents
					if (frequentVocabularyDictionary.contains(theTerm)) {
						frequentVocabularyDictionary
								.removeFromDictionary(theTerm);
					}

					// if the term is not already in the list of already-learnt,
					// then add it there
					if (!alreadyLearntDictionary.contains(theTerm)) {
						alreadyLearntDictionary.addToDictionary(theTerm);
					}
				}
				saveHelperDictionaries();
				updateVocabularyHistory();
			}
		};

		for (int i = 0; i < historyVocabularyMI.length; i++) {
			historyVocabularyMI[i] = new TermMenuItem("-");
			historyVocabularyMenu.add(historyVocabularyMI[i]);
			historyVocabularyMI[i].addMouseListener(historyVocabularyListener);
		}

		// ----- help menu settings
		// ---------------------------------------------------------------------
		helpMenu = new JMenu(helpMenuString);
		aboutMI = new JMenuItem(aboutMenuString);
		aboutMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showMessage(ABOUT_MESSAGE, false);
			}
		});
		menuBar.add(helpMenu);
		helpMenu.add(aboutMI);

		// ----- usage menu item settings
		// ---------------------------------------------------------------
		usageMI = new JMenuItem(usageMenuString);
		usageMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				File readmeFile = new File("readme.txt");

				if (true) {
					try {
						BufferedReader br = new BufferedReader(
								new InputStreamReader(new FileInputStream(
										readmeFile), encoding));

						StringBuilder builder = new StringBuilder();
						String line = br.readLine();

						while ((line != null)) {
							builder.append(line);
							builder.append("\n");
							line = br.readLine();
						}

						JFrame w = new JFrame(usageMenuString);
						JTextArea ta = new JTextArea(20, 20);
						w.setIconImage(image);
						w.getContentPane().add(new JScrollPane(ta));
						ta.setFont(new Font("Courier New", Font.BOLD, 12));
						ta.setText(builder.toString());
						ta.setCaretPosition(0);
						ta.setEditable(false);
						w.setSize(715, 700);
						w.setLocation((Toolkit.getDefaultToolkit()
								.getScreenSize().width - w.getWidth()) / 2,
								thisFrame.getHeight());
						w.setVisible(true);
					} catch (IOException ex) {
					}
				}
			}
		});
		helpMenu.add(usageMI);

		// ----- check for updates settings
		// ------------------------------------------------------------
		checkUpdatesMI = new JMenuItem(checkForUpdatesMenuItemString);
		checkUpdatesMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkForUpdatesOnline(false);
			}
		});
		helpMenu.add(checkUpdatesMI);

		// ----- expand menu button settings
		// ------------------------------------------------------------
		menuButton = new JButton("Menu");
		menuButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						menuBarVisible = !menuBarVisible;
						menuBar.setVisible(menuBarVisible);

						dispose();
						setUndecorated(!isUndecorated());
						pack();
						setVisible(true);
					}
				});
			}
		});

		KeyAdapter keyAdapter = new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				if ((e.getKeyCode() == KeyEvent.VK_A)
						&& (e.getModifiers() == InputEvent.CTRL_MASK)) {
					addToDictionary();
				} else if ((e.getKeyCode() == KeyEvent.VK_X)
						&& (e.getModifiers() == InputEvent.CTRL_MASK)) {
					exitMI.doClick();
				} else if ((e.getKeyCode() == KeyEvent.VK_S)
						&& (e.getModifiers() == InputEvent.CTRL_MASK)) {
					saveMI.doClick();
				} else if (e.getKeyCode() == KeyEvent.VK_SLASH) {
					answerTextField.setText("");
					help();
				} else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					if (answerTextField.getText().length() > 0) {
						checkAnswer(answerTextField.getText());
					}
				} else if ((e.getKeyCode() == KeyEvent.VK_E)
						&& (e.getModifiers() == InputEvent.CTRL_MASK)) {
					askExtendedQuestion();
				} else if (helpTask != null) {
					helpTask.pause(3);
				}
			}
		};

		answerTextField.addKeyListener(keyAdapter);
		questionLabel.addKeyListener(keyAdapter);
		helpButton.addKeyListener(keyAdapter);
		menuButton.addKeyListener(keyAdapter);

		MyMouseListener mouseListener = new MyMouseListener(this) {
			@Override
			public void mouseClicked(MouseEvent e) {
				if ((e.getSource().getClass() != JButton.class)
						&& (e.getClickCount() == 2)) {
					help();
				}
			}
		};

		questionLabel.addMouseListener(mouseListener);
		questionLabel.addMouseMotionListener(mouseListener);

		answerTextField.addMouseListener(mouseListener);
		answerTextField.addMouseMotionListener(mouseListener);

		helpButton.addMouseListener(mouseListener);
		helpButton.addMouseMotionListener(mouseListener);

		pauseButton.addMouseListener(mouseListener);
		pauseButton.addMouseMotionListener(mouseListener);
		
		menuButton.addMouseListener(mouseListener);
		menuButton.addMouseMotionListener(mouseListener);

		filenameFilter = new FilenameFilter() {
			public boolean accept(File dir, String name) {
				if (name.endsWith(".txt")) {
					return true;
				}

				return false;
			}
		};

		JPanel standardPanel = new JPanel();
		standardPanel.setLayout(new GridBagLayout());

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 0.5;
		c.gridx = 0;
		c.gridy = 0;
		standardPanel.add(questionLabel, c);
		c.weightx = 0.5;
		c.gridx = 1;
		standardPanel.add(answerTextField, c);
		c.weightx = 0;
		c.gridx = 2;
		standardPanel.add(helpButton, c);
		c.weightx = 0;
		c.gridx = 3;
		standardPanel.add(pauseButton, c);
		c.gridx = 4;
		standardPanel.add(menuButton, c);

		UIManager.put("ToolTip.background", new Color(250,250,120));
		
		standardPanel.setBorder(new BevelBorder(BevelBorder.RAISED));
		getContentPane().add(standardPanel, BorderLayout.CENTER);
		setAlwaysOnTop(true);
		pack();

		loadConfiguration();

		getDictionary(true);
		setVisible(true);
	}

	/**
	 * The main method.
	 * 
	 * @param args
	 *            arguments of the program, the first argument is the dictionary
	 *            file name
	 */
	public static void main(String[] args) {
		if (args.length == 0) {
			new Reminder(null);
		} else {
			new Reminder(args[0]);
		}
	}

	/**
	 * Check for updates online
	 * 
	 * @param silent
	 *            decides if message is presented
	 */
	private void checkForUpdatesOnline(final boolean silent) {
		new Thread() {
			@Override
			public void run() {
				try {
					URL url = new URL("http://code.google.com/p/wordsreminder/");
					URLConnection connection = url.openConnection();
					InputStream urlData = connection.getInputStream();
					Scanner s = new Scanner(urlData);
					s.findWithinHorizon("Words Reminder", 0);

					String version = s.next();
					version = version.substring(0, version.lastIndexOf("."));

					if (!version.equals(currentVersion)) {
						showMessage(updateToNewVersionString.replaceAll("X",
								version), false);
					} else {
						if (!silent) {
							showMessage(applicationIsUpToDateString, false);
						}
					}

					lastUpdateCheck = new Date();
					saveConfiguration();
				} catch (Exception e1) {
					e1.printStackTrace();
					showMessage(cannotPerforUpToDateCheckString, false);
					saveConfiguration();
				}
			}
		}.start();
	}

	/**
	 * Adds a term to the dictionary (if it supports adding new terms).
	 */
	private void addToDictionary() {
		if (!addingToDictionary) {
			addingToDictionary = true;
			new Thread(new Runnable() {
				public void run() {
					String w = JOptionPane.showInputDialog(thisFrame,
							enterTermString, enterTermExampleString);

					while ((w != null) && !w.equals("")
							&& !w.equals(enterTermExampleString)) {
						dictionary.addToDictionary(w);
						w = null;

						if (addToDictionaryMode == ADD_MANY_WORDS) {
							w = JOptionPane.showInputDialog(thisFrame,
									enterTermString, enterTermExampleString);
						}
					}

					addingToDictionary = false;
				}
			}).start();
		}
	}

	/**
	 * Asks a new question. It can be the same question, in case a wrong answer
	 * has been given.
	 * 
	 * @param theSameQuestion
	 *            true if the same question should be asked, false if a new one.
	 */
	private synchronized void askQuestion(boolean theSameQuestion) {
		if (!theSameQuestion) {
			getTermFromVocabulary();

			if ((questionsAsked == 0) && (showExtendedQuestionDelay > 0)) {
				askExtendedQuestion();
			}
		}

		String currentQuestionPresented = currentQuestion;
		if (currentQuestion.length() >= 60) {
			currentQuestionPresented = currentQuestion.substring(0, Math.min(
					60, currentQuestion.length()));
			currentQuestionPresented = currentQuestionPresented + "...";
		}
		questionLabel.setText("   " + currentQuestionPresented + "   ");
		answerTextField.setText("");
		answerTextField.setEditable(true);
		answerTextField.setBackground(Color.white);
		currentHintPosition = 0;
		questionsAsked++;
		refreshing = false;

		pack();
		setPosition();

		if (showHelpDelay > 0) {
			if (helpTask != null) {
				helpTask.cancel();
				helpTask = null;
			}

			answerTextField.setShowTime(true);
			timer.schedule(helpTask = new HelpTask(), 1000, 1000);
		}
	}

	private void setPosition() {
		if (center) {
			int questionLabelWidth = questionLabel.getWidth();
			int answerTextFieldWidth = answerTextField.getWidth();
			int helpBtnWidth = helpButton.getWidth();
			int menuBtnWidth = menuButton.getWidth();

			setLocation((Toolkit.getDefaultToolkit().getScreenSize().width
					- answerTextFieldWidth - helpBtnWidth - menuBtnWidth)
					/ 2 - questionLabelWidth + 30, getLocation().y);
		} else
			setLocation(0, 0);

	}

	/**
	 * Helper method used to get a term from vocabulary (dictionary). This
	 * method does not return anything, it just initializes currentTerm and
	 * other variables.
	 */
	private void getTermFromVocabulary() {
		ITerm term = null;

		IDictionary d;

		if (((questionsAsked % 5) == 0)
				&& (frequentVocabularyDictionary != null)
				&& (!frequentVocabularyDictionary.isEmpty())) {
			d = frequentVocabularyDictionary;
		} else {
			d = dictionary;
		}

		term = d.getRandomTerm();

		if (term == null) {
			showMessage(dictionaryIsEmptyErrorString, true);
		}

		int counter = 0;

		while (alreadyLearntDictionary.contains(term)) {
			term = d.getRandomTerm();

			// if 100 terms continuously are found in the already-learnt list it
			// means that
			// either all terms, or at least too many terms, are marked as
			// already-learnt
			// it is necessary to reset the list and delete the already-learnt
			// file
			if (counter == 100) {
				askIfResetAlreadyLearnt();
			}

			counter++;
		}

		if (firstLanguageMI.isSelected()) {
			currentQuestion = term.getWord();
			currentQuestionType = FIRST_LANGUAGE;
		} else if (secondLanguageMI.isSelected()) {
			currentQuestion = getOneOfTranslationsToAsk(term);
			currentQuestionType = SECOND_LANGUAGE;
		} else {
			currentQuestionType = random.nextInt(2) + 1;

			if (currentQuestionType == FIRST_LANGUAGE) {
				currentQuestion = term.getWord();
			} else {
				currentQuestion = getOneOfTranslationsToAsk(term);
			}
		}

		currentTerm = term;
	}

	/**
	 * If this method is called it means that either all terms, or too many
	 * terms, are marked as already-learnt. It is necessary to reset the list.
	 * First we ask the user if he or she agrees to do that, but if not, the
	 * program must be closed.
	 */
	private void askIfResetAlreadyLearnt() {
		int choice = JOptionPane.showConfirmDialog(thisFrame,
				tooManyAlreadyLearntLongInfo, "Problem",
				JOptionPane.YES_NO_OPTION);

		if (choice == JOptionPane.OK_OPTION) {
			alreadyLearntDictionary.removeAll();

			File alreadyLearntFile = new File("already-learnt.txt");

			if (alreadyLearntFile.exists()) {
				alreadyLearntFile.delete();
			}

			updateVocabularyHistory();
		} else {
			System.exit(0);
		}
	}

	/**
	 * Checks answer to the question.
	 * 
	 * @param text
	 *            the answer text
	 */
	private void checkAnswer(String text) {
		if ((currentQuestionType == FIRST_LANGUAGE)
				|| (currentQuestionType == EXTENDED)) {
			if (currentTerm.checkTranslation(answerTextField.getText())) {
				right(true);
			} else {
				wrong();
			}
		} else {
			if (currentTerm.getWord().equals(text)) {
				right(true);
			} else {
				wrong();
			}
		}
	}

	/**
	 * Gets the dictionary from the factory.
	 * 
	 * @param exitIfFails
	 *            if true then when loading fails the program exits
	 */
	private void getDictionary(boolean exitIfFails) {
		IDictionary d = DictionaryFactory.getDictionary(this,
				dictionaryFileName, encoding);

		if (d == null) {
			loadDictionary(exitIfFails);
		} else {
			dictionary = d;
			setTitle(titleString + dictionaryFileName);
		}

		File frequentVocabularyFile = new File("frequent.txt");

		if (frequentVocabularyFile.exists()) {
			frequentVocabularyDictionary = DictionaryFactory.getDictionary(
					this, "frequent.txt", encoding);
		} else {
			frequentVocabularyDictionary = DictionaryFactory
					.createEmptyDictionary(this, "frequent.txt", encoding);
		}

		File alreadyLearntFile = new File("already-learnt.txt");

		if (alreadyLearntFile.exists()) {
			alreadyLearntDictionary = DictionaryFactory.getDictionary(this,
					"already-learnt.txt", encoding);
		} else {
			alreadyLearntDictionary = DictionaryFactory.createEmptyDictionary(
					this, "already-learnt.txt", encoding);
		}

		if (dictionary != null) {
			askQuestion(false);
		}
	}

	/**
	 * Translates all texts from menu to another language.
	 * 
	 * @param isEnglish
	 *            true if english, false if polish.
	 */
	private void getLanguageBundle(boolean isEnglish) {
		if (isEnglish) {
			loadDictionaryString = "Load dictionary";
			addToDictionaryString = "Add to dictionary";
			enterTermString = "Enter word with translation";
			enterTermExampleString = "word = translation";
			saveChangesString = "Save changes";
			exitString = "Save and exit";
			fileMenuString = "File";
			optionsMenuString = "Options";
			alwaysOnTopString = "Always on top";
			askOnlyFirstLanguageString = "Ask only of first language";
			askOnlySecondLanguageString = "Ask only of second language";
			askBothLanguagesString = "Ask of both languages";
			changeLanguageMenuString = "Menu po polsku";
			showNoHelpString = "Do not show help";
			historyVocabularyMenuString = "History";
			aboutMenuString = "About this program";
			usageMenuString = "Help";
			codingUTF8String = "Encoding UTF-8";
			codingWindows1250String = "Encoding Windows1250";
			helpMenuString = "Help";
			totalAnsweredQuestionsString = "Answered questions: ";
			correctAnswersString = "Correct answers: ";
			incorrectAnswersString = "Incorrect answers: ";
			dictionaryFileErorString1 = "Not a dictionary file '";
			dictionaryFileErorString2 = "' or file corrupted. \nCorrect it or load another dictionary file. \n";
			badFormatString1 = "Bad format, dictionary term should have the following format:\nword = translation, other translation\n'";
			badFormatString2 = "' is not a correct dictionary term.";
			lineNrString = "Line ";
			titleString = "Words Reminder - ";
			showHelpNowString = "Show help now";
			centerString = "Center the window                               ";
			goodAnswerString = "Correct: ";
			goodAnswerString2 = "Perfect :-))) ";
			badAnswerString = "Incorrect answer, try again!";
			badAnswerString2 = "Some of the answers are not correct, try again.";
			configurationLoadingError = "Error in configuration file. Delete the 'config.properties' file.";
			dictionaryIsEmptyErrorString = "Dictionary file is empty.";
			noExtendedFilesErrorString = "You chose option to ask only of extended, but there are no extended files in \"extended\" directory.\n"
					+ "Try setting the \"typeOfTest\" option in config.properties file to \"vocabularyOnly\", or delete the file, or create some extended files in \"extended\" directory.";
			checkExtendedString = "Check";
			helpExtendedString = "Help";
			exitExtendedString = "Exit";
			nextExtendedString = "Next";
			extendedQuestionFrequencyString = "Ask extended question every X minute(s)";
			notShowExtendedQuestionsString = "Do not ask extended questions";
			showHelpAfterXSecondsString = "Show help after X second(s)";
			showExtendedQuestionNowString = "Ask extended question now";
			updateToNewVersionString = "There is a new version, X. Please visit the official site\n"
					+ "http://code.google.com/p/wordsreminder/ and download the new version.";
			applicationIsUpToDateString = "Application is up to date.";
			cannotPerforUpToDateCheckString = "Cannot connect to the official site. Check your internet connection and try again.";
			checkForUpdatesMenuItemString = "Check for updates online";
			helpDurationXString = "Help display duration: X second(s)";
			tooManyAlreadyLearntLongInfo = "There are too many already-learnt words. The list of already-learnt words must be cleared. Do you agree? \n"
					+ "If you choose \"No\" the program will have to be closed.";
			helpTooltipString = "<html><body>Left-click: Answer<br>Right-click: Hint</body></html>";
			extendedQuestionCategoryFieldTitle = "Extended questions category (tense)";
			extendedQuestionCategoryTooltip = "Enter the category or tense name";
			languageIsEnglish = true;
		} else {
			loadDictionaryString = "Zaaduj sownik";
			addToDictionaryString = "Dodaj do sownika";
			enterTermString = "Wprowad sowo i tumaczenie";
			enterTermExampleString = "sowo = tumaczenie";
			saveChangesString = "Zachowaj zmiany";
			exitString = "Zapisz i zakocz";
			fileMenuString = "Plik";
			optionsMenuString = "Opcje";
			alwaysOnTopString = "Zawsze na wierzchu";
			askOnlyFirstLanguageString = "Odpytuj tylko z pierwszego jzyka";
			askOnlySecondLanguageString = "Odpytuj tylko z drugiego jzyka";
			askBothLanguagesString = "Odpytuj z obu jzykw";
			changeLanguageMenuString = "Menu in english";
			showNoHelpString = "Nie pokazuj pomocy";
			historyVocabularyMenuString = "Historia";
			aboutMenuString = "Informacje o programie";
			codingUTF8String = "Kodowanie UTF-8";
			codingWindows1250String = "Kodowanie Windows1250";
			helpMenuString = "Pomoc";
			usageMenuString = "Pomoc";
			totalAnsweredQuestionsString = "Odpowiedziano na pyta: ";
			correctAnswersString = "Odopowiedzi poprawnych: ";
			incorrectAnswersString = "Odpowiedzi bdnych: ";
			dictionaryFileErorString1 = "Brakuje pliku sownika '";
			dictionaryFileErorString2 = "' lub plik ten jest uszkodzony. \nPopraw lub zaaduj inny plik sownika. \n";
			badFormatString1 = "Zy format, wpis w sowniku musi wyglda nastpujco:\nsowo = tumaczenie, inne tumaczenie\n'";
			badFormatString2 = "' nie jest poprawnym wpisem do sownika.";
			lineNrString = "Linia ";
			titleString = "Odpytywacz Swek - ";
			showHelpNowString = "Poka pomoc teraz";
			centerString = "Ustawiaj okienko porodku ekranu";
			goodAnswerString = "Dobrze: ";
			goodAnswerString2 = "Doskonale :-)))";
			badAnswerString = "Za odpowied, sprbuj jeszcze raz!";
			badAnswerString2 = "Niektre odpowiedzi s bdne, sprbuj jeszcze raz.";
			configurationLoadingError = "Bd w pliku konfiguracyjnym. Skasuj plik 'config.properties'.";
			dictionaryIsEmptyErrorString = "Plik sownika jest pusty";
			noExtendedFilesErrorString = "Wybrae opcj zadawania rozszerzonych pyta, ale nie ma adnych plikw pyta rozszerzonych w katalogu \"extended\".\n"
					+ "Sprbuj ustawi opcj \"typeOfTest\" w pliku config.properties na \"vocabularyOnly\", albo skasowa ten plik, albo utworzy pliki pyta rozszerzonych w katalogu \"extended\".";
			checkExtendedString = "Sprawd";
			helpExtendedString = "Pomoc";
			exitExtendedString = "Wyjd";
			nextExtendedString = "Nastpne";
			extendedQuestionFrequencyString = "Zadawaj pytania rozszerzone co X minut(y)";
			notShowExtendedQuestionsString = "Nie zadawaj pyta rozszerzonych";
			showHelpAfterXSecondsString = "Pokazuj pomoc po X sekundach";
			showExtendedQuestionNowString = "Zadaj rozszerzone pytanie";
			updateToNewVersionString = "Jest nowa wersja, X. Odwied oficjaln stron\n"
					+ "http://code.google.com/p/wordsreminder/ i pobierz now wersj.";
			applicationIsUpToDateString = "Wersja aplikacji jest aktualna.";
			cannotPerforUpToDateCheckString = "Nie mog poczy si z oficjaln stron programu. Sprawd poczenie z internetem i sprbuj ponownie.";
			checkForUpdatesMenuItemString = "Sprawd aktualizacje online";
			helpDurationXString = "Czas pokazywania pomocy: X sekund(y)";
			tooManyAlreadyLearntLongInfo = "Zbyt wiele sw znajduje si na licie \"dobrze znanych sw\". Lista musi by wyczyszczona. Zgadzasz si? \n"
					+ "Jeli wybierzesz \"Nie\" program bdzie musia by zamknity.";
			helpTooltipString = "<html><body>Left-click: Pomoc<br>Right-click: Podpowied</body></html>";
			extendedQuestionCategoryFieldTitle = "Kategoria pyta rozszerzonych (czas gr.)";
			extendedQuestionCategoryTooltip = "Wprowad nazw kategorii lub czasu";
			languageIsEnglish = false;
		}
	}

	/**
	 * Gets random term translation.
	 * 
	 * @param term
	 *            term to be analyzed
	 * 
	 * @return one of the terms translations, random generated
	 */
	private String getOneOfTranslationsToAsk(ITerm term) {
		String ask;
		String[] translations = term.getTranslation();

		if (translations.length == 1) {
			ask = translations[0];
		} else {
			int r = random.nextInt(translations.length);
			ask = translations[r];
		}

		return ask;
	}

	/**
	 * Shows help.
	 */
	private synchronized void help() {
		if (refreshing) {
			if (refreshTask != null) {
				refreshTask.run();
				refreshTask.cancel();
			}
		} else {
			String help = "";

			if ((currentQuestionType == FIRST_LANGUAGE)
					|| (currentQuestionType == EXTENDED)) {
				help = currentTerm.getTranslationAsOneString();
			} else {
				help = currentTerm.getWord();
			}

			answerTextField.setText(help);
			answerTextField.setEditable(false);

			setPosition();

			refresh(false);
		}
	}

	/**
	 * Shows hint - one letter from the answer.
	 */
	private synchronized void hint() {
		if (refreshing) {
			if (refreshTask != null) {
				refreshTask.run();
				refreshTask.cancel();
			}
		} else {
			String hint = "";

			if ((currentQuestionType == FIRST_LANGUAGE)
					|| (currentQuestionType == EXTENDED)) {
				hint = currentTerm.getTranslationAsOneString();
			} else {
				hint = currentTerm.getWord();
			}

			currentHintPosition++;

			// check if index is not out of bounds
			if (currentHintPosition < hint.length()) {

				// ignore white spaces
				while (hint.charAt(currentHintPosition - 1) == ' ')

					if (currentHintPosition < (hint.length() - 1)) {
						currentHintPosition++;
					}

				hint = hint.substring(0, currentHintPosition);

				answerTextField.setText(hint);
				answerTextField.setCaretPosition(hint.length());

			} else {
				right(false);
			}
		}
	}

	/**
	 * Loads configuration of the program if supplied. The configuration is in
	 * the config.properties file.
	 */
	private void loadConfiguration() {
		File configFile = new File(configFileName);

		if ((wantedDictionaryFileName != null)
				&& !wantedDictionaryFileName.equals("")) {
			dictionaryFileName = wantedDictionaryFileName;
		}

		if (configFile.exists()) {
			try {
				Properties props = new Properties();
				String t;

				props.load(new FileInputStream(configFile));

				t = (String) props.get("dictionaryFileName");

				if ((t != null) && !t.equals("")) {
					dictionaryFileName = t;
				}

				t = (String) props.get("termSeparator");

				if ((t != null) && !t.equals("")) {
					termSeparator = t;
				}

				t = (String) props.get("translationSeparator");

				if ((t != null) && !t.equals("")) {
					translationSeparator = t;
				}

				t = (String) props.get("extendedSeparatorFirst");

				if ((t != null) && !t.equals("")) {
					extendedSeparatorFirst = t;
				}

				t = (String) props.get("extendedSeparatorSecond");

				if ((t != null) && !t.equals("")) {
					extendedSeparatorSecond = t;
				}

				t = (String) props.get("centerWindow");

				if ((t != null) && !t.equals("")) {
					if (t.equals("true")) {
						center = true;
					} else {
						center = false;
					}

					centerMI.setSelected(center);
				}

				t = (String) props.get("showExtendedQuestionDelay");

				if ((t != null) && !t.equals("")) {
					showExtendedQuestionDelay = Integer.parseInt(t);
					extendedQuestionFrequencySMI
							.setValue(showExtendedQuestionDelay);

					if (showExtendedQuestionDelay > 0) {
						extendedQuestionFrequencySMI
								.setTitle(extendedQuestionFrequencyString
										.replaceAll(
												"X",
												Integer
														.toString(showExtendedQuestionDelay)));
					} else {
						extendedQuestionFrequencySMI
								.setTitle(notShowExtendedQuestionsString);
						extendedQuestionFrequencySMI.setForeground(Color.red);
					}
				}

				t = (String) props.get("language");

				if ((t != null) && !t.equals("")) {
					if (t.equals("english")) {
						languageIsEnglish = true;
					} else {
						languageIsEnglish = false;
					}

					getLanguageBundle(languageIsEnglish);
					refreshGui();
				}

				t = (String) props.get("whichLanguageToAsk");

				if ((t != null) && !t.equals("")) {
					if (t.equals("first")) {
						firstLanguageMI.setSelected(true);
					} else if (t.equals("second")) {
						secondLanguageMI.setSelected(true);
					} else if (t.equals("both")) {
						bothLanguagesMI.setSelected(true);
					}
				}

				t = (String) props.get("showHelpDelay");

				if ((t != null) && !t.equals("")) {
					showHelpDelay = Integer.parseInt(t);
					showHelpDelaySMI.setValue(showHelpDelay);

					if (showHelpDelay > 0) {
						showHelpDelaySMI.setTitle(showHelpAfterXSecondsString
								.replaceAll("X", Integer
										.toString(showHelpDelay)));
					} else {
						showHelpDelaySMI.setTitle(showNoHelpString);
					}
				}

				t = (String) props.get("helpDuration");

				if ((t != null) && !t.equals("")) {
					helpDuration = Integer.parseInt(t);
					helpDurationSMI.setValue(helpDuration);

					if (helpDuration <= 0) {
						helpDuration = 3;
					}

					helpDurationSMI.setTitle(helpDurationXString.replaceAll(
							"X", Integer.toString(helpDuration)));
				}

				t = (String) props.get("lastUpdateCheck");

				if ((t != null) && !t.equals("")) {
					DateFormat df = DateFormat
							.getDateInstance(DateFormat.SHORT);
					lastUpdateCheck = df.parse(t);

					Calendar todayCalendar = GregorianCalendar.getInstance();
					Calendar lastUpdateCheckCalendar = GregorianCalendar
							.getInstance();
					lastUpdateCheckCalendar.setTime(lastUpdateCheck);

					long diffMillis = todayCalendar.getTimeInMillis()
							- lastUpdateCheckCalendar.getTimeInMillis();
					long diffDays = diffMillis / (24 * 60 * 60 * 1000);

					if (diffDays > 21) {
						checkForUpdatesOnline(true);
					}
				} else {
					checkForUpdatesOnline(true);
				}
			} catch (Exception e) {
				e.printStackTrace();
				showMessage(configurationLoadingError, true);
			}
		} else {
			checkForUpdatesOnline(true);
		}
	}

	/**
	 * Loads another dictionary.
	 * 
	 * @param exitIfFails
	 *            if true then when loading fails the program exits
	 */
	public void loadDictionary(boolean exitIfFails) {
		JFileChooser chooser = new JFileChooser(lastPath);

		chooser.setFileFilter(new FileFilter() {
			@Override
			public boolean accept(File file) {
				if (file.getName().contains(".txt") || file.isDirectory()) {
					return true;
				}

				return false;
			}

			@Override
			public String getDescription() {
				return "*.txt";
			}
		});

		int returnVal = chooser.showOpenDialog(thisFrame);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			dictionaryFileName = chooser.getSelectedFile().getAbsolutePath();
			lastPath = chooser.getSelectedFile().getPath();

			// delete frequent and already-learnt files
			File frequentVocabularyFile = new File("frequent.txt");

			if (frequentVocabularyFile.exists()) {
				frequentVocabularyFile.delete();
			}

			File alreadyLearntFile = new File("already-learnt.txt");

			if (alreadyLearntFile.exists()) {
				alreadyLearntFile.delete();
			}

			// continue with the new dictionary
			getDictionary(exitIfFails);
		} else if (exitIfFails) {
			System.exit(0);
		}
	}

	/**
	 * Refreshes the program. Cancels the help task, updates history and asks
	 * new question.
	 * 
	 * @param theSame
	 *            the pararemter for askQuestion method, true if the same
	 *            question should be asked, false if a new one.
	 */
	private synchronized void refresh(final boolean theSame) {
		refreshing = true;

		if (helpTask != null) {
			helpTask.cancel();
			helpTask = null;
		}

		refreshTask = new TimerTask() {
			@Override
			public void run() {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (!theSame) {
							addToVocabularyHistory(currentTerm);
						}

						setIcon(0);
						askQuestion(theSame);
					}
				});
			}
		};
		timer.schedule(refreshTask, helpDuration * 1000);
	}

	/**
	 * Refreshes gui after changes of the menu language.
	 */
	private void refreshGui() {
		fileMenu.setText(fileMenuString);
		optionsMenu.setText(optionsMenuString);
		historyVocabularyMenu.setText(historyVocabularyMenuString);
		helpMenu.setText(helpMenuString);
		loadDictionaryMI.setText(loadDictionaryString);
		addToDictionaryMI.setText(addToDictionaryString);
		saveMI.setText(saveChangesString);
		exitMI.setText(exitString);
		alwaysOnTopMI.setText(alwaysOnTopString);
		languageMI.setText(changeLanguageMenuString);
		firstLanguageMI.setText(askOnlyFirstLanguageString);
		secondLanguageMI.setText(askOnlySecondLanguageString);
		bothLanguagesMI.setText(askBothLanguagesString);

		utf8MI.setText(codingUTF8String);
		cp1250MI.setText(codingWindows1250String);
		aboutMI.setText(aboutMenuString);
		showHelpNowMI.setText(showHelpNowString);
		centerMI.setText(centerString);
		showExtendedQuestionNowMI.setText(showExtendedQuestionNowString);
		checkUpdatesMI.setText(checkForUpdatesMenuItemString);
		usageMI.setText(usageMenuString);
		helpDurationSMI.setTitle(helpDurationXString.replaceAll("X", Integer
				.toString(helpDuration)));
		helpButton.setToolTipText(helpTooltipString);
		extendedQuestionCategory
				.setToolTipText(extendedQuestionCategoryTooltip);
		extendedQuestionCategory
				.setBorderTitle(extendedQuestionCategoryFieldTitle);

		if (extendedCheckBtn != null) {
			extendedCheckBtn.setText(checkExtendedString);
			extendedHelpBtn.setText(helpExtendedString);
			extendedExitBtn.setText(exitExtendedString);
			extendedNextBtn.setText(nextExtendedString);
		}

		if (showExtendedQuestionDelay > 0) {
			extendedQuestionFrequencySMI
					.setTitle(extendedQuestionFrequencyString.replaceAll("X",
							Integer.toString(showExtendedQuestionDelay)));
		} else {
			extendedQuestionFrequencySMI
					.setTitle(notShowExtendedQuestionsString);
		}

		if (showHelpDelay > 0) {
			showHelpDelaySMI.setTitle(showHelpAfterXSecondsString.replaceAll(
					"X", Integer.toString(showHelpDelay)));
		} else {
			showHelpDelaySMI.setTitle(showNoHelpString);
		}

		setTitle(titleString + dictionaryFileName);
	}

	/**
	 * Reacts for a right answer.
	 */
	private synchronized void right(boolean showCorrectString) {
		if (!refreshing) {
			String reason = "";

			if ((currentQuestionType == FIRST_LANGUAGE)
					|| (currentQuestionType == EXTENDED)) {
				reason = currentTerm.getTranslationAsOneString();
			} else {
				reason = currentTerm.getWord();
			}

			Color backgroundColor;
			String toPresent = "";
			if (showCorrectString) {
				toPresent = goodAnswerString + reason;
				backgroundColor = Color.green;
			} else {
				toPresent = reason;
				backgroundColor = Color.lightGray;
			}
			answerTextField.setText(toPresent);
			answerTextField.setEditable(false);
			answerTextField.setBackground(backgroundColor);

			rightAnswers++;
			refresh(false);
		}
	}

	/**
	 * Saves configuration of the program.
	 */
	private void saveConfiguration() {
		if (dictionary != null) {
			dictionary.save();
		}

		saveHelperDictionaries();

		Toolkit.getDefaultToolkit().beep();

		try {
			File configFile = new File(configFileName);
			Properties props = new Properties();
			props.put("dictionaryFileName", dictionaryFileName);
			props.put("termSeparator", termSeparator);
			props.put("translationSeparator", translationSeparator);
			props.put("extendedSeparatorFirst", extendedSeparatorFirst);
			props.put("extendedSeparatorSecond", extendedSeparatorSecond);

			if (center) {
				props.put("centerWindow", "true");
			} else {
				props.put("centerWindow", "false");
			}

			if (languageIsEnglish) {
				props.put("language", "english");
			} else {
				props.put("language", "polish");
			}

			props.put("showExtendedQuestionDelay", Integer
					.toString(showExtendedQuestionDelay));

			if (firstLanguageMI.isSelected()) {
				props.put("whichLanguageToAsk", "first");
			} else if (secondLanguageMI.isSelected()) {
				props.put("whichLanguageToAsk", "second");
			} else {
				props.put("whichLanguageToAsk", "both");
			}

			props.put("showHelpDelay", Integer.toString(showHelpDelay));
			props.put("helpDuration", Integer.toString(helpDuration));

			String dateString = DateFormat.getDateInstance(DateFormat.SHORT)
					.format(lastUpdateCheck);
			props.put("lastUpdateCheck", dateString);

			props.store(new FileOutputStream(configFile), null);
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
		}
	}

	private void saveHelperDictionaries() {
		if ((frequentVocabularyDictionary != null)
				&& !frequentVocabularyDictionary.isEmpty()) {
			frequentVocabularyDictionary.save();
		} else {
			File frequentVocabularyFile = new File("frequent.txt");

			if (frequentVocabularyFile.exists()) {
				frequentVocabularyFile.delete();
			}
		}

		if ((alreadyLearntDictionary != null)
				&& !alreadyLearntDictionary.isEmpty()) {
			alreadyLearntDictionary.save();
		} else {
			File alreadyLearntFile = new File("already-learnt.txt");

			if (alreadyLearntFile.exists()) {
				alreadyLearntFile.delete();
			}
		}
	}

	/**
	 * Sets the program icon (rotated).
	 * 
	 * @param percent
	 *            the percent of the wheel (rotation).
	 */
	private void setIcon(int percent) {
		Graphics2D g2d = null;

		if (imageIcon == null) {
			image = new BufferedImage(50, 50, BufferedImage.TYPE_INT_ARGB);
			imageIcon = new ImageIcon(image);
		}

		g2d = (Graphics2D) image.getGraphics();
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
				RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		g2d.setColor(Color.RED);
		g2d.fillRect(0, 0, 50, 50);
		g2d.setColor(Color.WHITE);
		g2d.setFont(iconFont);
		g2d.drawString("WR", 5, 35);

		int w = image.getWidth(null);
		int h = image.getHeight(null);

		AffineTransform affineTransform = new AffineTransform();
		// rotate with the anchor point as the mid of the image
		affineTransform.rotate(Math.toRadians(percent), w / 2, h / 2);

		g2d.transform(affineTransform);
		g2d.drawImage(image, 0, 0, Color.black, this);

		setIconImage(image);
	}

	/**
	 * Shows a message and terminates if needed.
	 * 
	 * @param message
	 *            message to be shown.
	 * @param exit
	 *            if this parameter is set to true then after showing the
	 *            message the program is terminated.
	 */
	void showMessage(String message, boolean exit) {
		JOptionPane.showMessageDialog(thisFrame, message, "Words Reminder",
				JOptionPane.INFORMATION_MESSAGE, imageIcon);

		if (exit) {
			System.exit(0);
		}
	}

	/**
	 * Adds a term to history of vocabulary.
	 * 
	 * @param term
	 *            term to be added to history of vocabulary
	 */
	private void addToVocabularyHistory(ITerm term) {
		historyVocabulary.addToHistory(term);
		updateVocabularyHistory();
	}

	/**
	 * Updates vocabulary history.
	 */
	private void updateVocabularyHistory() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				for (int i = 0; i < historyVocabulary.size(); i++) {
					historyVocabularyMI[i].setTerm(historyVocabulary.get(i));
					historyVocabularyMI[i].setText(historyVocabulary.get(i)
							.toString());

					if (frequentVocabularyDictionary.contains(historyVocabulary
							.get(i))) {
						historyVocabularyMI[i].setForeground(Color.red);
					} else if (alreadyLearntDictionary
							.contains(historyVocabulary.get(i))) {
						historyVocabularyMI[i].setForeground(Color.green);
					} else {
						historyVocabularyMI[i].setForeground(Color.blue);
					}
				}
			}
		});
	}

	/**
	 * Reacts to a wrong answer.
	 */
	private synchronized void wrong() {
		if (!refreshing) {
			answerTextField.setText(badAnswerString);
			answerTextField.setEditable(false);
			answerTextField.setBackground(Color.red);

			wrongAnswers++;
			refresh(true);
		}
	}

	/**
	 * Schedules extended question.
	 */
	private void scheduleExtendedQuestion() {
		if (showExtendedQuestionTimerTask != null) {
			showExtendedQuestionTimerTask.cancel();
		}

		showExtendedQuestionTimerTask = new TimerTask() {
			@Override
			public void run() {
				askExtendedQuestion();
			}
		};
		timer.schedule(showExtendedQuestionTimerTask,
				showExtendedQuestionDelay * 60000,
				showExtendedQuestionDelay * 60000);
	}

	/**
	 * Asks extended question.
	 */
	private void askExtendedQuestion() {
		// sentences building questions removed in version 1.5.1
		// only grammar question available now

		askGrammarQuestion(50);
	}

	/**
	 * Helper method used to get a term from extendeds. This method does not
	 * return anything, it just initializes currentTerm and other variables.
	 */
	private void askGrammarQuestion(int counter) {
		// sentences building questions removed in version 1.5.1
		// only grammar questions available now
		if (grammarQuestionOngoing) {
			return;
		}

		if (counter == 0)
			return;

		grammarQuestionOngoing = true;

		File dir = new File("extended");
		Category cat = null;

		if (dir.exists()) {
			File[] children = dir.listFiles(filenameFilter);

			if ((children == null) || (children.length == 0)) {
				// Either dir does not exist or is not a directory
			} else {
				Random r = new Random();

				// Get filename of file or directory
				File file = children[r.nextInt(children.length)];

				cat = new Category();
				cat.readFromFile(file.getAbsolutePath(), encoding);
			}
		}

		if (cat != null) {

			if (extendedQuestionCategory.getText() != null
					&& !extendedQuestionCategory.getText().equals("")) {
				Subcategory subcategory = cat
						.getSubcategory(extendedQuestionCategory.getText());
				if (subcategory == null) {
					grammarQuestionOngoing = false;
					askGrammarQuestion(counter - 1);
				} else
					showGrammarQuestion(subcategory);
			} else { // any category
				Subcategory subcategory = cat.getRandomTense();

				if (subcategory == null) {
					showMessage(noExtendedFilesErrorString + "2", true);
				} else {
					showGrammarQuestion(subcategory);
				}
			}
		}
	}

	/**
	 * Helper method used to get a term from extendeds. This method does not
	 * return anything, it just initializes currentTerm and other variables.
	 */
	/*
	 * private void askSentenceQuestion() { if (sentenceQuestionOngoing ||
	 * grammarQuestionOngoing) { return; }
	 * 
	 * sentenceQuestionOngoing = true;
	 * 
	 * File dir = new File("extended/sentences"); File randomSentencesFile =
	 * null;
	 * 
	 * if (dir.exists()) { File[] children = dir.listFiles(filenameFilter);
	 * 
	 * if ((children == null) || (children.length == 0)) { // Either dir does
	 * not exist or is not a directory } else { Random r = new Random(); // Get
	 * filename of file or directory File file =
	 * children[r.nextInt(children.length)];
	 * 
	 * if (file != null) { randomSentencesFile = file; } } }
	 * 
	 * if (randomSentencesFile != null) { if ((currentSentecesFileName == null) ||
	 * !currentSentecesFileName.equals(randomSentencesFile .getName())) { //
	 * first time currentSentecesFileName =
	 * randomSentencesFile.getAbsolutePath(); sentencesDictionary =
	 * DictionaryFactory.getDictionary(this, currentSentecesFileName, encoding); } }
	 * 
	 * if (sentencesDictionary != null) { ITerm term =
	 * sentencesDictionary.getRandomTerm(); String question = term.getWord();
	 * String answer = term.getTranslationAsOneString();
	 * showSentenceQuestion(question, answer); } }
	 */

	/**
	 * Shows sentences question.
	 * 
	 * @param question
	 *            TODO: DOCUMENT ME!
	 * @param answer
	 *            TODO: DOCUMENT ME!
	 */
	private void showSentenceQuestion(final String question, final String answer) {
		final JFrame frame = new JFrame(question);
		frame.setIconImage(image);

		JPanel panel = new JPanel();
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;

		final JLabel label = new JLabel("  " + question + "  ");
		MyMouseListener mml = new MyMouseListener(frame);
		label.addMouseListener(mml);
		label.addMouseMotionListener(mml);
		label.setFont(textFont);
		label.setForeground(Color.blue);

		StringTokenizer tokenizer = new StringTokenizer(answer);
		JTextField tf;

		gridbag.setConstraints(label, c);
		panel.add(label);

		/**
		 * Helper class.
		 */
		class Pair2 {
			/** The text field. */
			JTextField tf;

			/** The label. */
			String word;

			/**
			 * Creates a new Pair object.
			 * 
			 * @param tf
			 *            the text field
			 * @param label
			 *            the label
			 */
			public Pair2(JTextField tf, String word) {
				this.tf = tf;
				this.word = word;
			}
		}

		final List<Pair2> words = new ArrayList<Pair2>();

		int i = 1;

		JPanel answerPanel = new JPanel();
		GridBagLayout gridbag2 = new GridBagLayout();
		answerPanel.setLayout(gridbag2);

		while (tokenizer.hasMoreTokens()) {
			String s = tokenizer.nextToken();

			c.gridx = i++;

			int len = s.length();

			if (len > 4) {
				len = len - (len / 4);
			}

			tf = new JTextField(len);
			tf.setFont(textFont);
			tf.setHorizontalAlignment(tf.CENTER);
			tf.setBorder(new CompoundBorder(BorderFactory.createMatteBorder(0,
					3, 0, 3, Color.white), BorderFactory.createMatteBorder(0,
					0, 3, 0, Color.black)));
			gridbag2.setConstraints(tf, c);
			answerPanel.add(tf);

			if (s.contains(",")) {
				s = s.substring(0, s.indexOf(","));

				JLabel l = new JLabel(" , ");
				l.setFont(textFont);
				l.setBackground(Color.white);
				l.setOpaque(true);
				c.gridx = i++;
				gridbag2.setConstraints(l, c);
				answerPanel.add(l);
			}

			if (s.contains(".")) {
				s = s.substring(0, s.indexOf("."));

				JLabel l = new JLabel(" . ");
				l.setFont(textFont);
				l.setBackground(Color.white);
				l.setOpaque(true);
				c.gridx = i++;
				gridbag2.setConstraints(l, c);
				answerPanel.add(l);
			}

			if (s.contains("?")) {
				s = s.substring(0, s.indexOf("?"));

				JLabel l = new JLabel(" ? ");
				l.setFont(textFont);
				l.setBackground(Color.white);
				l.setOpaque(true);
				c.gridx = i++;
				gridbag2.setConstraints(l, c);
				answerPanel.add(l);
			}

			words.add(new Pair2(tf, s));
		}

		answerPanel.setBorder(BorderFactory.createLineBorder(Color.black));
		answerPanel.setBackground(Color.white);
		c.gridx = 1;
		c.gridy = 0;
		c.gridwidth = c.REMAINDER;
		c.weightx = 1;
		gridbag.setConstraints(answerPanel, c);
		panel.add(answerPanel);

		// show random word
		Random r = new Random();
		int k = r.nextInt(words.size());
		Pair2 p2 = words.get(k);
		p2.tf.setText(p2.word);
		p2.tf.setEditable(false);
		p2.tf.setFocusable(false);
		p2.tf.setBackground(Color.white);

		extendedCheckBtn = new JButton(checkExtendedString);
		extendedCheckBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				boolean res = true;

				for (int i = 0; i < words.size(); i++) {
					Pair2 p = words.get(i);

					if (!p.word.equalsIgnoreCase(p.tf.getText().trim())) {
						label.setForeground(Color.red);
						res = false;
					} else {
						p.tf.setBackground(Color.green);
						p.tf.setEditable(false);
						label.setForeground(Color.black);
					}
				}

				if (res) {
					showMessage(goodAnswerString2, false);
					frame.dispose();
					sentenceQuestionOngoing = false;
				} else {
					showMessage(badAnswerString2, false);
				}
			}
		});

		extendedExitBtn = new JButton(exitExtendedString);
		extendedExitBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				sentenceQuestionOngoing = false;
				frame.dispose();
			}
		});

		extendedHelpBtn = new JButton(helpExtendedString);
		extendedHelpBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				for (int i = 0; i < words.size(); i++) {
					Pair2 p = words.get(i);

					p.tf.setText(p.word);
					p.tf.setEditable(false);
					p.tf.setBackground(Color.white);
					label.setForeground(Color.black);
				}

				extendedCheckBtn.setEnabled(false);
				extendedHelpBtn.setEnabled(false);
			}
		});

		extendedNextBtn = new JButton(nextExtendedString);
		extendedNextBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				sentenceQuestionOngoing = false;
				frame.dispose();
				askExtendedQuestion();
			}
		});

		JPanel buttonsPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c2 = new GridBagConstraints();
		c2.gridx = 0;
		c2.gridy = 0;
		c2.weightx = 1;
		c2.fill = c2.HORIZONTAL;
		buttonsPanel.add(extendedCheckBtn, c2);
		c2.gridx = 1;
		c2.gridy = 0;
		c2.weightx = 0;
		buttonsPanel.add(extendedHelpBtn, c2);
		c2.gridx = 2;
		c2.gridy = 0;
		buttonsPanel.add(extendedNextBtn, c2);
		c2.gridx = 3;
		c2.gridy = 0;
		buttonsPanel.add(extendedExitBtn, c2);

		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = c.REMAINDER;
		c.weightx = 1;
		gridbag.setConstraints(buttonsPanel, c);
		panel.add(buttonsPanel);
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createLineBorder(Color.black), BorderFactory
				.createEmptyBorder(1, 1, 1, 1)));

		frame.getContentPane().add(panel);
		frame.setResizable(false);
		frame.setAlwaysOnTop(true);
		frame.setUndecorated(true);
		frame.pack();

		frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent arg0) {
				sentenceQuestionOngoing = false;
				frame.dispose();
			}
		});

		frame.setVisible(true);
		frame.setLocation(
				(Toolkit.getDefaultToolkit().getScreenSize().width - frame
						.getWidth()) / 2, thisFrame.getHeight());
	}

	/**
	 * Shows extended question.
	 * 
	 * @param subcategory
	 *            the category to be shown
	 */
	private void showGrammarQuestion(final Subcategory subcategory) {
		String title;

		if (subcategory.getParentName() != null) {
			title = subcategory.getParentName() + ", " + subcategory.getName();
		} else {
			title = subcategory.getName();
		}

		final JFrame frame = new JFrame(title);
		frame.setIconImage(image);

		JPanel panel = new JPanel();
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 1;

		JLabel label;

		/**
		 * Local class for grouping text field with label.
		 */
		class Pair {
			/** The text field. */
			JTextField tf;

			/** The label. */
			JLabel label;

			/**
			 * Creates a new Pair object.
			 * 
			 * @param tf
			 *            the text field
			 * @param label
			 *            the label
			 */
			public Pair(JTextField tf, JLabel label) {
				this.tf = tf;
				this.label = label;
			}
		}

		JTextField tf;
		final List<Pair> rows = new ArrayList<Pair>();
		int i = 0;
		label = new JLabel(" " + title + " ", JLabel.CENTER);

		MyMouseListener mml = new MyMouseListener(frame);
		label.addMouseListener(mml);
		label.addMouseMotionListener(mml);
		label.setFont(textFont);
		label.setForeground(Color.blue);
		c.gridwidth = 2;
		gridbag.setConstraints(label, c);
		panel.add(label);

		c.gridwidth = 1;

		for (; i < subcategory.size(); i++) {
			PrefixTerm term = (PrefixTerm) subcategory.getTerm(i);
			c.gridx = 0;
			c.gridy = i + 1;
			label = new JLabel("  " + term.getWord() + "  ", JLabel.RIGHT);
			label.setFont(textFont);
			gridbag.setConstraints(label, c);
			panel.add(label);
			c.gridx = 1;
			tf = new JTextField(20);
			tf.setFont(textFont);
			rows.add(new Pair(tf, label));
			gridbag.setConstraints(tf, c);
			panel.add(tf);
		}

		extendedCheckBtn = new JButton(checkExtendedString);
		extendedCheckBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				boolean res = true;

				for (int i = 0; i < subcategory.size(); i++) {
					PrefixTerm term = (PrefixTerm) subcategory.getTerm(i);
					Pair p = rows.get(i);

					if (!term.checkTranslation(p.tf.getText())) {
						p.label.setForeground(Color.red);
						res = false;
					} else {
						p.tf.setBackground(Color.green);
						p.tf.setEditable(false);
						p.label.setForeground(Color.black);
					}
				}

				if (res) {
					showMessage(goodAnswerString2, false);
					frame.dispose();
					grammarQuestionOngoing = false;
				} else {
					showMessage(badAnswerString2, false);
				}
			}
		});

		extendedExitBtn = new JButton(exitExtendedString);
		extendedExitBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				grammarQuestionOngoing = false;
				frame.dispose();
			}
		});

		extendedHelpBtn = new JButton(helpExtendedString);
		extendedHelpBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				for (int i = 0; i < subcategory.size(); i++) {
					PrefixTerm term = (PrefixTerm) subcategory.getTerm(i);
					Pair p = rows.get(i);

					p.tf.setText(term.getTranslationAsOneString());
					p.tf.setEditable(false);
					p.label.setForeground(Color.black);
				}

				extendedCheckBtn.setEnabled(false);
				extendedHelpBtn.setEnabled(false);
			}
		});

		extendedNextBtn = new JButton(nextExtendedString);
		extendedNextBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				grammarQuestionOngoing = false;
				frame.dispose();
				askExtendedQuestion();
			}
		});

		JPanel buttonsPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c2 = new GridBagConstraints();
		c2.gridx = 0;
		c2.gridy = 0;
		c2.weightx = 1;
		c2.fill = c2.HORIZONTAL;
		buttonsPanel.add(extendedCheckBtn, c2);
		c2.gridx = 1;
		c2.gridy = 0;
		c2.weightx = 0;
		buttonsPanel.add(extendedHelpBtn, c2);
		c2.gridx = 2;
		c2.gridy = 0;
		buttonsPanel.add(extendedNextBtn, c2);
		c2.gridx = 3;
		c2.gridy = 0;
		buttonsPanel.add(extendedExitBtn, c2);

		c.gridx = 0;
		c.gridy = i + 1;
		c.gridwidth = 2;
		c.weightx = 1;
		gridbag.setConstraints(buttonsPanel, c);
		panel.add(buttonsPanel);
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createLineBorder(Color.black), BorderFactory
				.createEmptyBorder(1, 1, 1, 1)));

		frame.getContentPane().add(panel);
		frame.setResizable(false);
		frame.setAlwaysOnTop(true);
		frame.setUndecorated(true);
		frame.pack();

		frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent arg0) {
				grammarQuestionOngoing = false;
				frame.dispose();
			}
		});

		frame.setVisible(true);
		frame.setLocation(
				(Toolkit.getDefaultToolkit().getScreenSize().width - frame
						.getWidth()) / 2, thisFrame.getHeight());
	}

	/**
	 * A MouseListener and MouseMotionListener implementation, used to enable
	 * dragging window after catching one of the components and enable
	 * double-click to show help.
	 */
	public class MyMouseListener implements MouseListener, MouseMotionListener {
		/** Initial location. */
		private Point location;

		/** Mouse event. */
		private MouseEvent pressed;

		/** TODO: DOCUMENT ME! */
		private JFrame frame;

		/**
		 * Creates a new MyMouseListener object.
		 * 
		 * @param frame
		 *            TODO: DOCUMENT ME!
		 */
		public MyMouseListener(JFrame frame) {
			super();
			this.frame = frame;
		}

		/**
		 * {@inheritDoc}
		 */
		public void mouseClicked(MouseEvent e) {
		}

		/**
		 * Handles mouse dragging.
		 * 
		 * @param me
		 *            mouse event
		 */
		public void mouseDragged(MouseEvent me) {
			location = frame.getLocation(location);

			int x = location.x - pressed.getX() + me.getX();
			int y = location.y - pressed.getY() + me.getY();
			frame.setLocation(x, y);
		}

		/**
		 * {@inheritDoc}
		 */
		public void mouseEntered(MouseEvent e) {
		}

		/**
		 * {@inheritDoc}
		 */
		public void mouseExited(MouseEvent e) {
		}

		/**
		 * {@inheritDoc}
		 */
		public void mouseMoved(MouseEvent e) {
		}

		/**
		 * {@inheritDoc}
		 */
		public void mousePressed(MouseEvent me) {
			pressed = me;
		}

		/**
		 * {@inheritDoc}
		 */
		public void mouseReleased(MouseEvent e) {
		}
	}

	/**
	 * This class performs actions connected to help showing.
	 */
	class HelpDurationChangeListener implements ChangeListener {
		/**
		 * Checks which of the menu items generated the event and acts
		 * accordingly, showing
		 * 
		 * @param e
		 *            the event
		 */
		public void stateChanged(ChangeEvent e) {
			if (helpDurationSMI.getValueIsAdjusting()) {
				return;
			}

			helpDuration = helpDurationSMI.getValue();

			if (helpDuration == 0) {
				helpDuration = 1;
				helpDurationSMI.setValue(helpDuration);
			}

			helpDurationSMI.setForeground(Color.blue);
			helpDurationSMI.setTitle(helpDurationXString.replaceAll("X",
					Integer.toString(helpDuration)));
		}
	}

	/**
	 * This class enables handling of the help feature.
	 */
	class HelpTask extends TimerTask {
		/** Counter. */
		private int count;

		/** Pause counter. */
		private int pause;

		// ~ Methods
		// ----------------------------------------------------------------------------------------------------
		/**
		 * Pauses the status bar.
		 */
		public void pause(int seconds) {
			pause = seconds;
		}
		
		public boolean isPaused(){
			return pause > 0;
		}

		/**
		 * Shows help and updates status bar.
		 */
		@Override
		public void run() {
			if (pause > 0) {
				pause--;

				return;
			}

			count++;

			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					answerTextField.setPart(count);

					if (count == showHelpDelay) {
						help();
					}
				}
			});
		}
	}

	/**
	 * This class performs actions connected to help showing.
	 */
	class ShowHelpChangeListener implements ChangeListener {
		/**
		 * Checks which of the menu items generated the event and acts
		 * accordingly, showing
		 * 
		 * @param e
		 *            the event
		 */
		public void stateChanged(ChangeEvent e) {
			if (showHelpDelaySMI.getValueIsAdjusting()) {
				return;
			}

			showHelpDelay = showHelpDelaySMI.getValue();

			if (showHelpDelay == 0) {
				showHelpDelaySMI.setForeground(Color.red);
				showHelpDelaySMI.setTitle(showNoHelpString);
			} else {
				showHelpDelaySMI.setForeground(Color.blue);
				showHelpDelaySMI.setTitle(showHelpAfterXSecondsString
						.replaceAll("X", Integer.toString(showHelpDelay)));
			}

			answerTextField.setShowTime(showHelpDelay > 0);

			if (helpTask != null) {
				helpTask.cancel();
				helpTask = null;
			}

			if (showHelpDelay > 0) {
				timer.schedule(helpTask = new HelpTask(), 1000, 1000);
			}
		}
	}

	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------
	/**
	 * This class extends JTextField to enable the status bar inside it.
	 */
	class TimerTextField extends JTextField {
		/** The part (percent) of the status bar. */
		private int part = 0;

		/** Color of the status bar. */
		private Color partColor = Color.red;

		/** Flag telling if the status bar is showed. */
		private boolean showTime = true;

		/** The default (start) number of columns of the text field. */
		private int startNumberOfColumns;

		/**
		 * Creates a new TimerTextField object.
		 * 
		 * @param columns
		 *            number of columns
		 */
		public TimerTextField(int columns) {
			super(columns);
			startNumberOfColumns = columns;
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public synchronized void paint(Graphics g) {
			try {
				if (g != null) {
					super.paint(g);

					if (showTime) {
						int width = getBounds().width;
						double onePartWidth = (double) width / showHelpDelay;
						g.setColor(partColor);
						g.fillRect(1, 1, (int) (part * onePartWidth), 2);
					}
				}
			} catch (RuntimeException e) {
			}
		}

		/**
		 * Sets the part of the status bar.
		 * 
		 * @param part
		 *            the part
		 */
		public synchronized void setPart(int part) {
			this.part = part;

			double percent = ((double) part / showHelpDelay) * 360;
			setIcon((int) percent);
			repaint();
		}

		/**
		 * Sets if the status bar should be showed.
		 * 
		 * @param show
		 *            if true then the status bar is showed, if false it is
		 *            hidden.
		 */
		public synchronized void setShowTime(boolean show) {
			showTime = show;
			setPart(0);
		}

		/**
		 * Sets the text and expands the text field to better fit the width of
		 * the text.
		 * 
		 * @param text
		 *            the text to be set
		 */
		@Override
		public synchronized void setText(String text) {
			// calculate width of text in pixels
			FontMetrics metrics = getGraphics().getFontMetrics(getFont());
			double width = metrics.stringWidth(text);

			// if this is too much than the actual size of the field
			if ((width + 5) > getWidth()) {
				// first resize the field, but max 2 times
				double n = (double) ((width + 5) / getColumnWidth());
				int columns = Math.min((int) Math.ceil(n),
						startNumberOfColumns * 2);
				setColumns(columns);
				// pack at once, so that the new size is known here
				thisFrame.pack();

				// recalculate the relation again
				double rel = (double) width / getWidth();

				// if text is still too big then cut off the ending
				if (rel > 0.98) {
					int len = (int) ((text.length() / rel) - 3);
					text = text.substring(0, len) + "...";
				}
			} else {
				// otherwise resize to the original
				setColumns(startNumberOfColumns);
				thisFrame.pack();
			}

			super.setText(text);
			setCaretPosition(0);
		}
	}
}

/**
 * MenuItem having its own ITerm.
 */
class TermMenuItem extends JMenuItem {
	/** The ITerm. */
	private ITerm term;

	/**
	 * Creates a new TermMenuItem object.
	 * 
	 * @param text
	 *            the initial text
	 */
	public TermMenuItem(String text) {
		super(text);
	}

	/**
	 * Gets the term.
	 * 
	 * @return the term
	 */
	public ITerm getTerm() {
		return term;
	}

	/**
	 * Sets the term.
	 * 
	 * @param term
	 *            term to be set
	 */
	public void setTerm(ITerm term) {
		this.term = term;
	}
}

/**
 * Slider menu item class.
 */
class SliderMenuItem extends JSlider implements MenuElement {
	/** Title border. */
	private TitledBorder titledBorder;

	/**
	 * Creates a new SliderMenuItem object.
	 * 
	 * @param min
	 *            minimum value
	 * @param max
	 *            maximum value
	 * @param value
	 *            initial value
	 * @param title
	 *            title
	 */
	public SliderMenuItem(int min, int max, int value, String title) {
		titledBorder = new TitledBorder(title + "  ");
		setBorder(new CompoundBorder(titledBorder, new EmptyBorder(5, 5, 5, 5)));

		setMajorTickSpacing(10);
		setMinorTickSpacing(5);
		setPaintLabels(true);
		setPaintTicks(true);
		setMinimum(min);
		setMaximum(max);
		setValue(value);
	}

	/**
	 * Sets the title of the title bar.
	 * 
	 * @param title
	 *            new title
	 */
	public void setTitle(String title) {
		titledBorder.setTitle(title);
	}

	/**
	 * {@inheritDoc}
	 */
	public void processMouseEvent(MouseEvent e, MenuElement[] path,
			MenuSelectionManager manager) {
		super.processMouseMotionEvent(e);
		super.processMouseEvent(e);
	}

	/**
	 * {@inheritDoc}
	 */
	public void processKeyEvent(KeyEvent e, MenuElement[] path,
			MenuSelectionManager manager) {
		super.processKeyEvent(e);
	}

	/**
	 * {@inheritDoc}
	 */
	public void menuSelectionChanged(boolean isIncluded) {
	}

	/**
	 * {@inheritDoc}
	 */
	public MenuElement[] getSubElements() {
		return new MenuElement[0];
	}

	/**
	 * {@inheritDoc}
	 */
	public Component getComponent() {
		return this;
	}
}

/**
 * Slider menu item class.
 */
class TextFieldMenuItem extends JTextField implements MenuElement {
	/** Title border. */
	private TitledBorder titledBorder;

	/**
	 * Creates a new SliderMenuItem object.
	 * 
	 * @param min
	 *            minimum value
	 * @param max
	 *            maximum value
	 * @param value
	 *            initial value
	 * @param title
	 *            title
	 */
	public TextFieldMenuItem(String title, String defaultText, Color background) {
		titledBorder = new TitledBorder(title + "  ");
		setBorder(new CompoundBorder(titledBorder, new EmptyBorder(5, 5, 5, 5)));
		setBackground(background);
		setText(defaultText);
		setFont(new Font("Courier New", Font.BOLD, 14));
	}

	/**
	 * Sets the title of the title bar.
	 * 
	 * @param title
	 *            new title
	 */
	public void setTitle(String title) {
		titledBorder.setTitle(title);
	}

	/**
	 * {@inheritDoc}
	 */
	public void processMouseEvent(MouseEvent e, MenuElement[] path,
			MenuSelectionManager manager) {
		super.processMouseMotionEvent(e);
		super.processMouseEvent(e);
	}

	/**
	 * {@inheritDoc}
	 */
	public void processKeyEvent(KeyEvent e, MenuElement[] path,
			MenuSelectionManager manager) {
		super.processKeyEvent(e);
	}

	/**
	 * {@inheritDoc}
	 */
	public void menuSelectionChanged(boolean isIncluded) {
	}

	/**
	 * {@inheritDoc}
	 */
	public MenuElement[] getSubElements() {
		return new MenuElement[0];
	}

	/**
	 * {@inheritDoc}
	 */
	public Component getComponent() {
		return this;
	}

	public void setBorderTitle(String title) {
		titledBorder = new TitledBorder(title + "  ");
		setBorder(new CompoundBorder(titledBorder, new EmptyBorder(5, 5, 5, 5)));
	}
}
