/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package wordsreminder;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Class used to handle grammar terms. It reads all tenses and build a list,
 * just like they were separate terms.
 * 
 * @author Jacek Koleyski
 */
public class Subcategory {
	// ~ Instance fields
	// ------------------------------------------------------------------------------------------------

	/** The parent name. */
	private String parentName;

	/** The tense name. */
	private String categoryName;

	/** The list of all grammar, handled as separate terms. */
	private List<ITerm> list = new ArrayList<ITerm>();

	public Subcategory(String parentName, String name) {
		this.parentName = parentName;
		this.categoryName = name;
	}

	// ~ Methods
	// --------------------------------------------------------------------------------------------------------

	/**
	 * Method returning a random grammar of a random tense.
	 * 
	 * @return a term being one of the grammar terms of one of the tenses
	 */
	public ITerm getRandomTerm() {
		Random r = new Random();

		return list.get(r.nextInt(list.size()));
	}

	public void add(ITerm term) {
		list.add(term);
	}

	public ITerm getTerm(int i) {
		return list.get(i);
	}

	public int size() {
		return list.size();
	}

	public String getName() {
		return categoryName;
	}

	public String getParentName() {
		return parentName;
	}

}
